<?php

require_once("PHPUnit/Framework.php" );
require_once("MyTestRunner.php");
require_once("MyTestFormatter.php");

function get_suite_from_file($suite, $file, $checkBasename=true) {
    $basename = basename($file);
    if ($checkBasename) {
        if (!preg_match('/.*test.*\./i', $basename)) {
            #print("File $basename doesn't look like a test file\n");
            return;
        }
    }
    # Always load only .inc or .php files
    if (!preg_match('/.*\.(?:php|inc)$/i', $basename)) {
        #print("File $basename doesn't look like a PHP file\n");
        return;
    }
    $before_classes = get_declared_classes();
    #print("**************** About to load file $file from dir " . getcwd() . "\n");
    $file_fs = ereg_replace('\\\\', '/', $file);
    #print("**************** Actually $file_fs... \n");
    require_once $file_fs;
    $after_classes = get_declared_classes();
    $new_classes = array_diff($after_classes, $before_classes);
    foreach ($new_classes as $class) {
        if (preg_match('/(?:T|\\bt)est(?:$|[^a-z])/i', $class)) {
            # print "Adding class $class from file $file ($basename)\n";
            $suite->addTestSuite($class);
        }
    }
}

function has_test_dir($dir_contents) {
    foreach ($dir_contents as $subdir) {
        if (preg_match('/^tests?$/i', $subdir)) {
            return $subdir;
        }
    }
    return null;
}
function is_test_dir($parent, $subdir) {
    return (preg_match('/^tests?$/i', $subdir)
            && is_dir("$parent/$subdir") );
}

function is_possible_subdir_name($dir) {
    return $dir != '.' && $dir != '..' && $dir != '.svn';
}

function get_suite_from_all_modules($suite, $directory, $subdir, $checkFilename) {
    $fullname = "$directory/$subdir";
    if (is_dir($fullname)) {
        $dh = opendir($fullname);
        if (!$dh) {
            return;
        }
        while (($file = readdir($dh)) != false) {
            if (is_possible_subdir_name($file)) {
                get_suite_from_all_modules($suite, $fullname, $file, $checkFilename);
            }
        }
        closedir($dh);
    } else {
        get_suite_from_file($suite, $fullname, $checkFilename);
    }
}

function get_suite_from_dir($directory) {
    // Find all tests under dir $directory...
    // Introspect to pull out the test classes
    if (!is_dir($directory)) {
        // Maybe it's a file
        if (!file_exists($directory)) {
            print("Can't work with \$directory=$directory\n");
            return $suite;
        }
        $suite = new PHPUnit_Framework_TestSuite();
        get_suite_from_file($suite, $directory, true);
        return $suite;
    } else {
        $suite = new PHPUnit_Framework_TestSuite();
        // Look for a 'test' or 'tests' dir
        $members = scandir($directory);
        if (has_test_dir($members)) {
            # Just pick out the testdirs and walk their contents
            foreach ($members as $subdir) {
                if (is_test_dir($directory, $subdir)) {
                    $curr_dir = getcwd();
                    chdir("$directory");
                    get_suite_from_all_modules($suite, ".", $subdir, false);
                    chdir($curr_dir);
                }
            }
        }
        $dh = opendir($directory);
        if (!$dh) {
            return $suite;
        }
        while (($file = readdir($dh)) != false) {
            if (is_possible_subdir_name($file)) {
                get_suite_from_all_modules($suite, $directory, $file, true);
            }
        }
        closedir($dh);
    }
    return $suite;
}

$dirname = null;
if ($argc <= 1) {
    $dirname = getcwd();
} else {
    $dirname = $argv[1];
}
#printf("\$dirname=%s\n", $dirname);
$suite = get_suite_from_dir($dirname);
if ($suite) {
    $runner = new tests_TestRunner( $suite );
    $formatter = new KoUnit_Formatter();
    $runner->addFormatter($formatter);
    $runner->run();
}

?>
